@@ -25,7 +25,6 @@
 
 #include "bz-addons-dialog.h"
 #include "bz-age-rating-dialog.h"
-#include "bz-app-permissions.h"
 #include "bz-app-size-dialog.h"
 #include "bz-app-tile.h"
 #include "bz-apps-page.h"
@@ -33,19 +32,17 @@
 #include "bz-context-tile.h"
 #include "bz-developer-badge.h"
 #include "bz-dynamic-list-view.h"
-#include "bz-env.h"
 #include "bz-error.h"
 #include "bz-fading-clamp.h"
 #include "bz-favorite-button.h"
 #include "bz-flatpak-entry.h"
 #include "bz-full-view.h"
-#include "bz-global-net.h"
 #include "bz-hardware-support-dialog.h"
 #include "bz-lazy-async-texture-model.h"
 #include "bz-license-dialog.h"
 #include "bz-releases-list.h"
-#include "bz-safety-dialog.h"
 #include "bz-safety-calculator.h"
+#include "bz-safety-dialog.h"
 #include "bz-screenshot-page.h"
 #include "bz-screenshots-carousel.h"
 #include "bz-section-view.h"
@@ -54,7 +51,6 @@
 #include "bz-state-info.h"
 #include "bz-stats-dialog.h"
 #include "bz-tag-list.h"
-#include "bz-util.h"
 
 struct _BzFullView
 {
@@ -216,7 +212,7 @@ is_zero (gpointer object,
 
 static gboolean
 is_positive (gpointer object,
-         int      value)
+             int      value)
 {
   return value > -1;
 }
@@ -247,6 +243,15 @@ is_empty (gpointer    object,
   return g_list_model_get_n_items (model) == 0;
 }
 
+static gboolean
+is_empty_string (gpointer    object,
+                 const char *str)
+{
+  if (str == NULL)
+    return TRUE;
+  return *str == '\0';
+}
+
 static gboolean
 logical_and (gpointer object,
              gboolean value1,
@@ -351,6 +356,24 @@ format_size (gpointer object, guint64 value)
   return g_strdup (size_str);
 }
 
+static  char *
+get_size_label (gpointer object,
+                gboolean is_installable)
+{
+   return g_strdup (is_installable ? _("Download") : _("Installed"));
+}
+
+static guint64
+get_size_type (gpointer object,
+               BzEntry *entry,
+               gboolean is_installable)
+{
+  if (entry == NULL)
+    return 0;
+
+  return is_installable ? bz_entry_get_size (entry) : bz_entry_get_installed_size (entry);
+}
+
 static char *
 format_size_tooltip (gpointer object, guint64 value)
 {
@@ -549,7 +572,7 @@ pick_license_warning (gpointer object,
 static char *
 format_other_apps_label (gpointer object, const char *developer)
 {
-  if (!developer || *developer == '\0' || strlen (developer) > 12)
+  if (!developer || *developer == '\0')
     return g_strdup (_ ("More Apps"));
   return g_strdup_printf (_ ("More Apps by %s"), developer);
 }
@@ -567,34 +590,45 @@ static char *
 format_leftover_label (gpointer object, const char *name, guint64 size)
 {
   g_autofree char *formatted_size = NULL;
+
   formatted_size = g_format_size (size);
   return g_strdup_printf (_ ("%s is not installed, but it still has <b>%s</b> of data present."), name, formatted_size);
 }
 
 static char *
 get_safety_rating_icon (gpointer object,
-                        BzEntry *entry)
+                        BzEntry *entry,
+                        int      index)
 {
-  BzImportance importance;
+  char        *icon       = NULL;
+  BzImportance importance = 0;
 
   if (entry == NULL)
     return g_strdup ("app-safety-unknown-symbolic");
 
-  importance = bz_safety_calculator_calculate_rating (entry);
+  if (index < 0 || index > 2)
+    return NULL;
 
-  switch (importance)
+  if (index == 0)
     {
-    case BZ_IMPORTANCE_UNIMPORTANT:
-    case BZ_IMPORTANCE_NEUTRAL:
-    case BZ_IMPORTANCE_INFORMATION:
-      return g_strdup ("app-safety-ok-symbolic");
-    case BZ_IMPORTANCE_WARNING:
-      return g_strdup ("app-safety-unknown-symbolic");
-    case BZ_IMPORTANCE_IMPORTANT:
-      return g_strdup ("app-safety-unsafe-symbolic");
-    default:
-      return g_strdup ("app-safety-unknown-symbolic");
+      importance = bz_safety_calculator_calculate_rating (entry);
+      switch (importance)
+        {
+        case BZ_IMPORTANCE_UNIMPORTANT:
+        case BZ_IMPORTANCE_NEUTRAL:
+          return g_strdup ("app-safety-ok-symbolic");
+        case BZ_IMPORTANCE_INFORMATION:
+        case BZ_IMPORTANCE_WARNING:
+          return NULL;
+        case BZ_IMPORTANCE_IMPORTANT:
+          return g_strdup ("dialog-warning-symbolic");
+        default:
+          return NULL;
+        }
     }
+
+  icon = bz_safety_calculator_get_top_icon (entry, index - 1);
+  return icon;
 }
 
 static char *
@@ -642,7 +676,7 @@ get_safety_rating_label (gpointer object,
     case BZ_IMPORTANCE_NEUTRAL:
       return g_strdup (_ ("Low Risk"));
     case BZ_IMPORTANCE_INFORMATION:
-      return g_strdup (_ ("Medium Risk"));
+      return g_strdup (_ ("Low Risk"));
     case BZ_IMPORTANCE_WARNING:
       return g_strdup (_ ("Medium Risk"));
     case BZ_IMPORTANCE_IMPORTANT:
@@ -940,12 +974,12 @@ static void
 safety_cb (BzFullView *self,
            GtkButton  *button)
 {
-  AdwDialog *dialog   = NULL;
+  AdwDialog *dialog = NULL;
 
   if (self->group == NULL)
     return;
 
-  dialog  = ADW_DIALOG (bz_safety_dialog_new (self->group));
+  dialog = ADW_DIALOG (bz_safety_dialog_new (self->group));
 
   adw_dialog_present (dialog, GTK_WIDGET (self));
 }
@@ -1033,8 +1067,6 @@ support_cb (BzFullView *self,
     }
 }
 
-
-
 static void
 install_addons_cb (BzFullView *self,
                    GtkButton  *button)
@@ -1239,6 +1271,7 @@ bz_full_view_class_init (BzFullViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, is_positive);
   gtk_widget_class_bind_template_callback (widget_class, is_null);
   gtk_widget_class_bind_template_callback (widget_class, is_empty);
+  gtk_widget_class_bind_template_callback (widget_class, is_empty_string);
   gtk_widget_class_bind_template_callback (widget_class, logical_and);
   gtk_widget_class_bind_template_callback (widget_class, is_longer);
   gtk_widget_class_bind_template_callback (widget_class, bool_to_string);
@@ -1246,6 +1279,7 @@ bz_full_view_class_init (BzFullViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, format_recent_downloads);
   gtk_widget_class_bind_template_callback (widget_class, format_recent_downloads_tooltip);
   gtk_widget_class_bind_template_callback (widget_class, format_size);
+  gtk_widget_class_bind_template_callback (widget_class, get_size_label);
   gtk_widget_class_bind_template_callback (widget_class, format_size_tooltip);
   gtk_widget_class_bind_template_callback (widget_class, age_rating_cb);
   gtk_widget_class_bind_template_callback (widget_class, format_age_rating);
@@ -1273,6 +1307,7 @@ bz_full_view_class_init (BzFullViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, dl_stats_cb);
   gtk_widget_class_bind_template_callback (widget_class, screenshot_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, size_cb);
+  gtk_widget_class_bind_template_callback (widget_class, get_size_type);
   gtk_widget_class_bind_template_callback (widget_class, formfactor_cb);
   gtk_widget_class_bind_template_callback (widget_class, safety_cb);
   gtk_widget_class_bind_template_callback (widget_class, run_cb);
