@@ -49,13 +49,13 @@ static GParamSpec *props[LAST_PROP] = { NULL };
 static gboolean invert_boolean (gpointer object,
                                 gboolean value);
 
-static char    *get_label_cb (gpointer object,
-                              BzEntry *entry);
+static char *get_label_cb (gpointer object,
+                           BzEntry *entry);
 
-static char    *get_license_info (gpointer object,
-                                  BzEntry *entry);
+static char *get_license_info (gpointer object,
+                               BzEntry *entry);
 
-static void     contribute_cb (BzLicenseDialog *self);
+static void contribute_cb (BzLicenseDialog *self);
 
 static void
 bz_license_dialog_dispose (GObject *object)
@@ -168,13 +168,17 @@ static char *
 get_label_cb (gpointer object,
               BzEntry *entry)
 {
-  const char *license  = NULL;
-  gboolean    is_floss = FALSE;
+  g_autofree char *license  = NULL;
+  gboolean         is_floss = FALSE;
 
   if (entry == NULL)
     return g_strdup ("");
 
-  g_object_get (entry, "is-floss", &is_floss, "project-license", &license, NULL);
+  g_object_get (
+      entry,
+      "is-floss", &is_floss,
+      "project-license", &license,
+      NULL);
 
   if (license == NULL || *license == '\0')
     return g_strdup (_ ("Unknown License"));
@@ -189,53 +193,13 @@ get_label_cb (gpointer object,
 }
 
 static char *
-get_license_info (gpointer object,
-                  BzEntry *entry)
+format_license_link (const char *license)
 {
-  const char      *license      = NULL;
-  gboolean         is_floss     = FALSE;
   g_autofree char *license_name = NULL;
   g_autofree char *license_url  = NULL;
-  g_autofree char *link         = NULL;
 
-  if (entry == NULL)
-    return g_strdup ("");
-
-  g_object_get (entry, "is-floss", &is_floss, "project-license", &license, NULL);
-
-  if (is_floss && bz_spdx_is_valid (license))
-    {
-      license_name = bz_spdx_get_name (license);
-      if (license_name == NULL || *license_name == '\0')
-        {
-          g_clear_pointer (&license_name, g_free);
-          license_name = g_strdup (license);
-        }
-
-      license_url = bz_spdx_get_url (license);
-      link        = g_strdup_printf ("<a href=\"%s\">%s</a>", license_url, license_name);
-
-      return g_strdup_printf (_ ("This app is developed in the open by an international community, "
-                                 "and released under the %s license.\n\n"
-                                 "You can participate and help make it even better."),
-                              link);
-    }
-
-  if (is_floss)
-    {
-      return g_strdup (_ ("This app is developed in the open by an international community.\n\n"
-                          "You can participate and help make it even better."));
-    }
-
-  if (license == NULL || *license == '\0')
-    return g_strdup (_ ("The license of this app is not known"));
-
-  if (bz_spdx_is_proprietary (license))
-    {
-      return g_strdup (_ ("This app is not developed in the open, so only its developers know how it works. "
-                          "It may be insecure in ways that are hard to detect, and it may change without oversight.\n\n"
-                          "You may or may not be able to contribute to this app."));
-    }
+  if (!bz_spdx_is_valid (license))
+    return g_strdup (license);
 
   license_name = bz_spdx_get_name (license);
   if (license_name == NULL || *license_name == '\0')
@@ -244,9 +208,56 @@ get_license_info (gpointer object,
       license_name = g_strdup (license);
     }
 
+  license_url = bz_spdx_get_url (license);
+  return g_strdup_printf ("<a href=\"%s\">%s</a>", license_url, license_name);
+}
+
+static char *
+get_license_info (gpointer object,
+                  BzEntry *entry)
+{
+  g_autofree char *license  = NULL;
+  gboolean         is_floss = FALSE;
+  g_autofree char *link     = NULL;
+
+  if (entry == NULL)
+    return g_strdup ("");
+
+  g_object_get (
+      entry,
+      "is-floss", &is_floss,
+      "project-license", &license,
+      NULL);
+
+  if (license == NULL || *license == '\0')
+    {
+      if (is_floss)
+        return g_strdup (_ ("This app is developed in the open by an international community.\n\n"
+                            "You can participate and help make it even better."));
+      else
+        return g_strdup (_ ("The license of this app is not known"));
+    }
+
+  if (is_floss)
+    {
+      link = format_license_link (license);
+      return g_strdup_printf (_ ("This app is developed in the open by an international community, "
+                                 "and released under the %s license.\n\n"
+                                 "You can participate and help make it even better."),
+                              link);
+    }
+
+  if (bz_spdx_is_proprietary (license))
+    {
+      return g_strdup (_ ("This app is not developed in the open, so only its developers know how it works. "
+                          "It may be insecure in ways that are hard to detect, and it may change without oversight.\n\n"
+                          "You may or may not be able to contribute to this app."));
+    }
+
+  link = format_license_link (license);
   return g_strdup_printf (_ ("This app is developed under the special license %s.\n\n"
                              "You may or may not be able to contribute to this app."),
-                          license_name);
+                          link);
 }
 
 static void
@@ -262,7 +273,7 @@ contribute_cb (BzLicenseDialog *self)
     return;
 
   first_url = g_list_model_get_item (share_urls, 1);
-  url = bz_url_get_url (first_url);
+  url       = bz_url_get_url (first_url);
 
   if (url != NULL && *url != '\0')
     g_app_info_launch_default_for_uri (url, NULL, NULL);
