@@ -1026,58 +1026,96 @@ parse_object (BzYamlParser  *self,
             {
               g_autofree char *anchor = NULL;
 
-              EXPECT (YAML_SCALAR_EVENT, "scalar value");
-              anchor = bz_maybe_strdup ((const char *) event->data.scalar.anchor);
-
-              switch (spec->value_type)
+              if (spec->value_type == G_TYPE_STRING &&
+                  event->type == YAML_MAPPING_START_EVENT)
                 {
-                case G_TYPE_BOOLEAN:
-                  vtype = G_VARIANT_TYPE_BOOLEAN;
-                  break;
-                case G_TYPE_INT:
-                  vtype = G_VARIANT_TYPE_INT32;
-                  break;
-                case G_TYPE_INT64:
-                  vtype = G_VARIANT_TYPE_INT64;
-                  break;
-                case G_TYPE_UINT:
-                  vtype = G_VARIANT_TYPE_UINT32;
-                  break;
-                case G_TYPE_UINT64:
-                  vtype = G_VARIANT_TYPE_UINT64;
-                  break;
-                case G_TYPE_DOUBLE:
-                case G_TYPE_FLOAT:
-                  vtype = G_VARIANT_TYPE_DOUBLE;
-                  break;
-                case G_TYPE_STRING:
-                default:
-                  vtype = G_VARIANT_TYPE_STRING;
-                  break;
-                }
+                  /* Handle optional translated strings */
+                  const char *const *langs   = NULL;
+                  g_autofree char   *english = NULL;
 
-              if (g_variant_type_equal (vtype, G_VARIANT_TYPE_STRING))
-                variant = g_variant_new_string ((const char *) event->data.scalar.value);
+                  anchor = bz_maybe_strdup ((const char *) event->data.mapping_start.anchor);
+
+                  langs = g_get_language_names ();
+                  for (;;)
+                    {
+                      g_autofree char *code = NULL;
+
+                      NEXT_EVENT ();
+                      if (event->type == YAML_MAPPING_END_EVENT)
+                        break;
+                      EXPECT (YAML_SCALAR_EVENT, "scalar key language code");
+                      if (variant != NULL)
+                        continue;
+
+                      code = g_strdup ((const char *) event->data.scalar.value);
+
+                      NEXT_EVENT ();
+                      EXPECT (YAML_SCALAR_EVENT, "scalar translated string");
+
+                      if (g_strv_contains (langs, code))
+                        variant = g_variant_new_string ((const char *) event->data.scalar.value);
+                      else if (g_strcmp0 (code, "en") == 0)
+                        english = g_strdup ((const char *) event->data.scalar.value);
+                    }
+
+                  if (variant == NULL)
+                    variant = g_variant_new_string (english != NULL ? english : "NULL");
+                }
               else
                 {
-                  variant = g_variant_parse (
-                      vtype,
-                      (const char *) event->data.scalar.value,
-                      NULL,
-                      NULL,
-                      &local_error);
-                  if (variant == NULL)
+                  EXPECT (YAML_SCALAR_EVENT, "scalar value");
+                  anchor = bz_maybe_strdup ((const char *) event->data.scalar.anchor);
+
+                  switch (spec->value_type)
                     {
-                      g_set_error (
-                          error,
-                          BZ_YAML_ERROR,
-                          BZ_YAML_ERROR_BAD_SCALAR,
-                          "Failed to parse scalar variant at line %zu, column %zu: %s",
-                          event->start_mark.line,
-                          event->start_mark.column,
-                          local_error->message);
-                      yaml_event_delete (event);
-                      return FALSE;
+                    case G_TYPE_BOOLEAN:
+                      vtype = G_VARIANT_TYPE_BOOLEAN;
+                      break;
+                    case G_TYPE_INT:
+                      vtype = G_VARIANT_TYPE_INT32;
+                      break;
+                    case G_TYPE_INT64:
+                      vtype = G_VARIANT_TYPE_INT64;
+                      break;
+                    case G_TYPE_UINT:
+                      vtype = G_VARIANT_TYPE_UINT32;
+                      break;
+                    case G_TYPE_UINT64:
+                      vtype = G_VARIANT_TYPE_UINT64;
+                      break;
+                    case G_TYPE_DOUBLE:
+                    case G_TYPE_FLOAT:
+                      vtype = G_VARIANT_TYPE_DOUBLE;
+                      break;
+                    case G_TYPE_STRING:
+                    default:
+                      vtype = G_VARIANT_TYPE_STRING;
+                      break;
+                    }
+
+                  if (g_variant_type_equal (vtype, G_VARIANT_TYPE_STRING))
+                    variant = g_variant_new_string ((const char *) event->data.scalar.value);
+                  else
+                    {
+                      variant = g_variant_parse (
+                          vtype,
+                          (const char *) event->data.scalar.value,
+                          NULL,
+                          NULL,
+                          &local_error);
+                      if (variant == NULL)
+                        {
+                          g_set_error (
+                              error,
+                              BZ_YAML_ERROR,
+                              BZ_YAML_ERROR_BAD_SCALAR,
+                              "Failed to parse scalar variant at line %zu, column %zu: %s",
+                              event->start_mark.line,
+                              event->start_mark.column,
+                              local_error->message);
+                          yaml_event_delete (event);
+                          return FALSE;
+                        }
                     }
                 }
 
